@@ -35,11 +35,11 @@
 #include <glib/gstdio.h>
 #include <gmodule.h>
 #include <pango/pango-utils.h>
-#include "gtkalias.h"
 #include "gtkimmodule.h"
 #include "gtkimcontextsimple.h"
 #include "gtkrc.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 /* Do *not* include "gtkprivate.h" in this file. If you do, the
  * correct_libdir_prefix() function below will have to move somewhere
@@ -95,7 +95,7 @@ gtk_im_module_load (GTypeModule *module)
 {
   GtkIMModule *im_module = GTK_IM_MODULE (module);
   
-  im_module->library = g_module_open (im_module->path, 0);
+  im_module->library = g_module_open (im_module->path, G_MODULE_BIND_LAZY);
   if (!im_module->library)
     {
       g_warning (g_module_error());
